options fullstimer;
options BUFSIZE=32k BUFNO=2k;
options compress=yes ;

/*programme qui construit les parts de marchs utilises pour la rgression finale*/
/*on ne calcule la taille du march, donc on utilise simplement les quantits */
/*One observation  : month , year, marque, model, segment, energy  */
/*three nests : segment/model/energy*/
/*Input Data : petrole.total and petrole2.instrument*/ 
/*Output Data : petrole2.tablespartstvared*/


libname petrole "V:\DG75-GD3E\DMSE\Petrole\tables travail";
libname petrole2 "u:\Petrole\tables travail";
  
 %macro nested_logit;
    
proc sql;
/* 1 model= marque*modele*energie2*segment  */
	create table model as
		select distinct mois, annee, marque, modele, segment, energie  
		from petrole.total  ; 
/* 1 : Sum of sales by model by  date * demo */
   create table ventesparmod_energ as
		select distinct annee, mois, demo, 
		segment,marque, modele, energie , 
		sum(volume*prix)/sum(volume) as prix_m,
		sum(volume*prixkilomlag3m)/sum(volume) as prixkilomlag3m_m,
		sum(volume*conso)/sum(volume) as conso_m,
		sum(volume*nb_portes=3)/sum(volume) as nbportes3,
		sum(volume*nb_portes=5)/sum(volume) as nbportes5,
		sum(volume*puis_kw)/sum(volume) as puis_kw 
		%do i=1 %to 5 ;, 
		sum(volume*claspuis&i.)/sum(volume) as  claspuis&i.  
		%end; 
		%do i=0 %to 6 ;, 
		sum(volume*clasco&i.)/sum(volume) as  clasco&i.  
		%end;  
		,sum(volume) as totmod_energ  
		from petrole.total 
		group by annee, mois, demo,
		 segment,marque, modele, energie 
        order by demo,annee, mois, segment ,marque, modele   ;

 
 
/*  2 : Sum of sales by model , to obtain the share of gasoline by model  */ 
  create table ventesparmodele as
		select distinct annee, mois, demo,  segment,marque, modele  ,
		sum(totmod_energ) as tot_mod  ,
		sum((energie='GAZOLE')*totmod_energ) /sum(totmod_energ) as partdiesel_mod,
		sum(conso_m*totmod_energ) /sum(totmod_energ) as conso_mod,
		sum(conso_m*totmod_energ*(energie='GAZOLE'))/sum(totmod_energ*(energie='GAZOLE')) 
				as consodies_mod
		from ventesparmod_energ 
		group by annee, mois, demo,  segment,marque, modele 
		order by demo,annee, mois,  segment,marque, modele ; 
 
 
/* 3 : sum of sales by segment, to obtain the share within segment */ 
  create table ventesparsegment as
		select distinct annee, mois, demo, segment  ,
		sum(tot_mod) as tot_seg  ,
		sum(partdiesel_mod*tot_mod) /sum(tot_mod) as partdiesel_seg,
		sum(conso_mod*tot_mod) /sum(tot_mod) as conso_seg,
		sum(consodies_mod*partdiesel_mod*tot_mod)/sum(partdiesel_mod*tot_mod ) 
				as consodies_seg
		from ventesparmodele 
		group by annee, mois, demo, segment 
		order by demo,annee, mois, segment;  
 

data parts_glob;
	merge ventesparmod_energ ventesparmodele;
	by demo annee  mois segment marque modele;
run;

data parts_glob;
	merge parts_glob  ventesparsegment ;
	by demo annee  mois segment ;
run;

/**ATTENTION: on ne peut pas sommer directement pour avoir les ventes totales par mois  partir de parts_glob***/
/*dans chaque segment autant de lignes que de modele avec le meme tot_seg : il faut retenir une ligne par segment pour ne pas estimer tot_seg^2 */
proc sql;
create table temp as select distinct 
annee,mois, demo , 
tot_seg,conso_seg,partdiesel_seg,consodies_seg,consodies_mod,
/*moyenne pondre par part du modele en diesel sur ensemble des diesel de la conso moyenne par modele / par version (ie conso moyenne des diesels)*/
		sum(conso_m*totmod_energ*(energie='GAZOLE'))/sum(totmod_energ *(energie='GAZOLE') ) as consodies_tot
from parts_glob
group by annee, mois, demo 
order by demo,annee, mois  ;

 
create table petrole2.partsglob_tottvared 
as select distinct 
annee,mois, demo,   consodies_tot,consodies_seg,consodies_mod,
        sum(tot_seg) as tot_tot,
        sum(tot_seg*conso_seg)/sum(tot_seg) as conso_moytot,
		sum(partdiesel_seg*tot_seg) /sum(tot_seg) as partdiesel_tot
        from temp 
group by annee, mois, demo 
order by demo,annee, mois  
;  

data parts_glob;merge parts_glob petrole2.partsglob_tottvared  ;by demo annee  mois  ;run;
proc sort data=parts_glob;
	by annee mois  marque modele segment energie    ;

data parts_glob;
	merge parts_glob(in=a) petrole2.instrumenttvared (in=b);
	by annee mois marque modele segment energie     ;
	if a and b;
run; 
proc sort data=parts_glob;by demo;
run;
%mend;
%nested_logit;

data petrole2.tablespartstvared (compress=yes);	
set parts_glob ;

/*deductible VAT for businesses*/
if (substr(left(demo),1,1)='1') and (energie='GAZOLE') then prixkilomlag3m_m=prixkilomlag3m_m*1.0392/1.196;
  
	log_parts= log(totmod_energ); 
	log_partsmod = log(totmod_energ / tot_mod);
	log_partsseg = log(tot_mod / tot_seg);
	log_partsseg2=log(totmod_energ/ tot_seg);
	log_prix = log(prix_m);  
	log_prixkilomlag3m = log(prixkilomlag3m_m);  
run; 
 
proc contents ;run;


proc sort data= petrole2.tablesparts; by demo ; run ; 

PROC DATASETS LIB=work KILL NOLIST;run;

proc freq data=  petrole2.tablespartstvared   ;
tables demo;
run; /*


                                                                        Frquence    Pctage
                                demo       Frquence     Pourcentage     cumule      cumul
                                
                                0_0_0_0        7050          1.62           7050       1.62
                                0_0_1_0       14931          3.43          21981       5.04
                                0_0_2_0       20859          4.79          42840       9.83
                                0_1_0_0        3668          0.84          46508      10.67
                                0_1_1_0        9235          2.12          55743      12.79
                                0_1_2_0       12041          2.76          67784      15.56
                                0_2_0_1       11910          2.73          79694      18.29
                                0_2_0_2        9634          2.21          89328      20.50
                                0_2_1_1       17488          4.01         106816      24.51
                                0_2_1_2       15086          3.46         121902      27.98
                                0_2_2_1       21185          4.86         143087      32.84
                                0_2_2_2       21489          4.93         164576      37.77
                                0_3_0_1       11044          2.53         175620      40.30
                                0_3_0_2       10761          2.47         186381      42.77
                                0_3_1_1       14785          3.39         201166      46.17
                                0_3_1_2       16389          3.76         217555      49.93
                                0_3_2_1       18666          4.28         236221      54.21
                                0_3_2_2       20140          4.62         256361      58.83
                                1_0_A         13890          3.19         270251      62.02
                                1_0_B         17816          4.09         288067      66.11
                                1_0_C         14300          3.28         302367      69.39
                                1_0_E         12097          2.78         314464      72.17
                                1_1_A          3278          0.75         317742      72.92
                                1_1_B          5285          1.21         323027      74.13
                                1_1_C          4616          1.06         327643      75.19
                                1_1_E          1511          0.35         329154      75.54
                                1_2_A         16146          3.71         345300      79.24
                                1_2_B         21648          4.97         366948      84.21
                                1_2_C         16355          3.75         383303      87.96
                                1_2_E         13416          3.08         396719      91.04
                                1_3_A         11715          2.69         408434      93.73
                                1_3_B         11977          2.75         420411      96.48
                                1_3_C         11850          2.72         432261      99.20
                                1_3_E          3489          0.80         435750     100.00

*/
  
proc means data=  petrole.tablesparts  cv std mean;
var log_prixkilomlag3m log_prix prixkilomlag3m_m prix_m log_parts log_partsmod log_partsseg;
run;
  

/**********************************/
/*pour construire une nouvelle variable dpendante permettant de tester l'impact de la taille du march*/

proc sort data= petrole.ventes_potentielles; by demo ; run ; 
 
data tablesparts2;	
	merge petrole.tablesparts(in=a) petrole.ventes_potentielles(in=b);
	by demo;
	if a and b; 
run;

proc means data=petrole.partsglob_tot max min p90;var tot_tot   ;class demo;run;
proc means data=petrole.ventes_potentielles max min p10 ;var ventes_potentielles;class demo;run;

data petrole.tablespartsnetext;	
	merge tablesparts2(in=a) petrole.partsglob_tot(in=b);
	by demo;
	if a and b; 
	/*log parts de marchs net du bien extrieur*/
	log_parts_netext= log(totmod_energ)-log(ventes_potentielles-tot_tot); 
run;

proc freq data=  petrole.tablespartsnetext   ;
tables demo;
run;
  
