/*First stage Tests: implementation of Sanderson, E. and Windmeijer, F. (2016), A Weak Instrument F-test in Linear IV Models with
Multiple Endogenous Variables, Journal of Econometrics 190(2), 212221.*/
/*input Table : petrole.tablespartstvared */
options fullstimer;
options BUFSIZE=32k BUFNO=2k;
options compress=yes ;
libname petrole "W:\@Encours\petrole\tables travail";
libname sorties "W:\@Encours\petrole\sorties";
libname sorttemp "W:\@Encours\petrole\sortiestemporaires";
 
data tablesfirststage ;set  petrole.tablespartstvared (where=((annee<2007 or mois<11) and modelcomm=1));  
length  marque0-marque15 segment0-segment4 date1-date58 3.;
datem=(annee-2003)*12+mois;
array date[58] date1-date58;
do i=1 to 58;
date[i]=(datem=i);
end;
 
array an[5] an1-an5;
do i=1 to 5;
 an[i]=(annee=2002+i);
end; 

array m[12] m1-m12;
do i=1 to 12;
 m[i]=(mois=i);
end; 

prof=(substr(left(demo),1,1)='1');

gasole=(energie='GAZOLE');
sqr_prixkilomlag3m_m=(prixkilomlag3m_m)**(0.5);
marque1=(marque='AUDI');
marque2=(marque='B.M.W.');
marque3=(marque='CITROEN');
marque4=(marque='FIAT');
marque5=(marque='FORD');
marque6=(marque='HYUNDAI');
marque7=(marque='MERCEDES');
marque8=(marque='NISSAN');
marque9=(marque='OPEL');
marque10=(marque='PEUGEOT');
marque11=(marque='SEAT');
marque12=(marque='SUZUKI');
marque13=(marque='TOYOTA');
marque14=(marque='VOLKSWAGEN');
marque15=(marque='RENAULT');
marque0=1-marque1-marque2-marque3-marque4-marque5-marque6-marque7-marque8-marque9-
marque10-marque11-marque12-marque13-marque14-marque15;
segment0=(segment = "TT-TERR");
segment1=(segment = "UTILIT");
segment2=(segment = "MONOSP"); 
segment3=(segment = "CITADINE"); 
segment4=(segment = "COMPACTE"); 
;
run;

%let list_exog= prixkilomlag3m_m claspuis2 claspuis3 claspuis4 claspuis5  clasco2 clasco3 clasco4 clasco5 clasco6
gasole marque0 marque1 marque2 marque3 marque4
marque5 marque6 marque7 marque8 marque9
marque10 marque11 marque12 marque13 marque14 segment0 segment1 segment2 segment3 segment4
nbportes5 nbportes3 an1 an2 an3 an4 m1 m2 m3 m4 m5 m6 m7 m8 m9 m10 m11;

%let list_endog=prix_m log_partsseg log_partsmod ;

%let list_inst=nb_model
totms_prixkilom_s totmps_prixkilom_s totpms_prixkilom_s totpmps_prixkilom_s
totms_claspuis_s2 totmps_claspuis_s2 totpms_claspuis_s2 totpmps_claspuis_s2
totms_claspuis_s3 totmps_claspuis_s3 totpms_claspuis_s3 totpmps_claspuis_s3
totms_claspuis_s4 totmps_claspuis_s4 totpms_claspuis_s4 totpmps_claspuis_s4
totms_claspuis_s5 totmps_claspuis_s5 totpms_claspuis_s5 totpmps_claspuis_s5
totms_clasco_s2 totmps_clasco_s2 totpms_clasco_s2 totpmps_clasco_s2
totms_clasco_s3 totmps_clasco_s3 totpms_clasco_s3 totpmps_clasco_s3
totms_clasco_s4 totmps_clasco_s4 totpms_clasco_s4 totpmps_clasco_s4
totms_clasco_s5 totmps_clasco_s5 totpms_clasco_s5 totpmps_clasco_s5
totms_clasco_s6 totmps_clasco_s6 totpms_clasco_s6 totpmps_clasco_s6
diff_prixkilomlag3mb diff_nbporte3b diff_nbporte5b diff_claspuis2b diff_claspuis3b diff_claspuis4b diff_claspuis5b
diff_clasco2b diff_clasco3b diff_clasco4b diff_clasco5b diff_clasco6b
cs_caoutch cs_fer cs_lcaoutch cs_lfer
difseg_cs_caoutch difseg_cs_lcaoutch difseg_cs_fer  difseg_cs_lfer
diff_cs_caoutchb diff_cs_ferb diff_cs_lcaoutchb diff_cs_lferb
;
 

/*Estimations en prenant en compte les autres variables (Angrist Pischke adapt, cf article Journal of econometrics)*/
%macro regfirststageAPmod(varendog=,datain=,dataout=,list_inst= ,cond=,list_exog=);
%let nb_varendog=%sysfunc(countw(&varendog.)); 
%let nb_inst=%sysfunc(countw(&list_inst.));  

%do k=1 %to 3;
%let varendog1=%scan(&varendog.,&k.);
%let varendog2=%sysfunc(prxchange(s/\b%sysfunc(tranwrd(&varendog1,%str( ),%str(\b|\b)))\b//,-1,&varendog));

/*Step 1  : to get residuals from the 2SLS regression*/
proc syslin data=&datain. 2sls out=resid ;&cond.;
instruments  &list_inst.   &list_exog. ;
endogenous %do j=1 %to 2 ; %scan(&varendog2.,&j.) %end;;
model &varendog1.=%do j=1 %to 2 ; %scan(&varendog2.,&j.) %end;  &list_exog. ;; 
output r= &varendog1.r;
run;

/*Step 2 :  Fischer test*/
proc reg data=resid;&cond.;
ods output Testanova=fisher;
model &varendog1.r  =  &list_inst.   &list_exog.;
test %do j=1 %to &nb_inst. ; %scan(&list_inst.,&j.) =%end; 0 ; 
run;
/*Degree of freedom corrections*/
data fvalue2&varendog1.;set  fisher(where=(source="Numerator"));
fvaluecorr&varendog1.=fvalue*&nb_inst./(&nb_inst.-2);
keep fvaluecorr&varendog1. demo;
run;  
%end; 
data &dataout;merge 
fvalue2%scan(&varendog.,1)  
fvalue2%scan(&varendog.,2)  
fvalue2%scan(&varendog.,3)  
;
by demo;
run;

%mend; 
  
option mprint;
%regfirststageAPmod(varendog=&list_endog., datain=tablesfirststage ,dataout=sortie, list_exog=&list_exog., list_inst=&list_inst., cond=by demo ); 
  
proc print data=sortie;run;
/* 27/12/2016
                                fvaluecorrprix_    fvaluecorrlog_    fvaluecorrlog_
              Obs     demo             m              partsseg          partsmod

                1    0_0_0_0        20.4420            53.3513           14.9938
                2    0_0_1_0        42.1961            58.3260           27.2819
                3    0_0_2_0        44.1558            49.2637           31.6507
                4    0_1_0_0        16.3476            36.4838            7.6261
                5    0_1_1_0        36.6434            53.5451           16.6517
                6    0_1_2_0        39.4334            54.8398           22.2471
                7    0_2_0_1        35.1066            68.1483           26.4873
                8    0_2_0_2        31.9356            71.3108           22.6156
                9    0_2_1_1        51.7568            61.4123           32.6404
               10    0_2_1_2        51.7404            62.4470           32.2993
               11    0_2_2_1        47.7544            60.7046           41.9384
               12    0_2_2_2        49.0495            53.6492           44.6591
               13    0_3_0_1        31.6596            71.4991           23.6408
               14    0_3_0_2        37.6451            73.3366           27.9019
               15    0_3_1_1        64.8218            71.1121           31.6798
               16    0_3_1_2        70.8544            63.9589           38.1015
               17    0_3_2_1        51.3077            58.9482           43.7454
               18    0_3_2_2        51.4806            55.6629           44.3282
               19    1_0_A          52.4334            31.9396           15.1586
               20    1_0_B          39.5158            26.2961           21.4837
               21    1_0_C          34.6307            31.6131           20.7671
               22    1_0_E          36.2984            36.4232           20.9952
               23    1_1_A          14.1664            23.0400           11.7357
               24    1_1_B          23.4862            25.2061           14.1968
               25    1_1_C          15.4412            11.4125           10.3705
               26    1_1_E          14.0952            11.5325            2.8914
               27    1_2_A          45.9496            43.2674           24.2059
               28    1_2_B          41.8890            39.9338           28.5747
               29    1_2_C          37.2478            41.9130           25.7957
               30    1_2_E          34.9028            30.5044           21.6317
               31    1_3_A          42.2016            44.7987           32.3586
               32    1_3_B          42.6386            43.4598           28.0160
               33    1_3_C          39.6835            38.7382           28.6363
               34    1_3_E          11.4750            14.8767            6.2873
*/
 